@@ -1,3 +1,4 @@
+use super::skip_pooling_allocator_tests;
 use anyhow::Result;
 use wasmtime::*;
 
@@ -11,11 +12,11 @@ fn successful_instantiation() -> Result<()> {
             table_elements: 10,
             ..Default::default()
         },
-        instance_limits: InstanceLimits {
-            count: 1,
-            memory_reservation_size: 1,
-        },
+        instance_limits: InstanceLimits { count: 1 },
     });
+    config.dynamic_memory_guard_size(0);
+    config.static_memory_guard_size(0);
+    config.static_memory_maximum_size(65536);
 
     let engine = Engine::new(&config)?;
     let module = Module::new(&engine, r#"(module (memory 1) (table 10 funcref))"#)?;
@@ -37,11 +38,11 @@ fn memory_limit() -> Result<()> {
             table_elements: 10,
             ..Default::default()
         },
-        instance_limits: InstanceLimits {
-            count: 1,
-            memory_reservation_size: 196608,
-        },
+        instance_limits: InstanceLimits { count: 1 },
     });
+    config.dynamic_memory_guard_size(0);
+    config.static_memory_guard_size(65536);
+    config.static_memory_maximum_size(3 * 65536);
 
     let engine = Engine::new(&config)?;
 
@@ -187,8 +188,11 @@ fn memory_guard_page_trap() -> Result<()> {
 }
 
 #[test]
-#[cfg_attr(target_arch = "aarch64", ignore)] // https://github.com/bytecodealliance/wasmtime/pull/2518#issuecomment-747280133
 fn memory_zeroed() -> Result<()> {
+    if skip_pooling_allocator_tests() {
+        return Ok(());
+    }
+
     let mut config = Config::new();
     config.allocation_strategy(InstanceAllocationStrategy::Pooling {
         strategy: PoolingAllocationStrategy::NextAvailable,
@@ -197,11 +201,11 @@ fn memory_zeroed() -> Result<()> {
             table_elements: 0,
             ..Default::default()
         },
-        instance_limits: InstanceLimits {
-            count: 1,
-            memory_reservation_size: 1,
-        },
+        instance_limits: InstanceLimits { count: 1 },
     });
+    config.dynamic_memory_guard_size(0);
+    config.static_memory_guard_size(0);
+    config.static_memory_maximum_size(65536);
 
     let engine = Engine::new(&config)?;
 
@@ -240,11 +244,11 @@ fn table_limit() -> Result<()> {
             table_elements: TABLE_ELEMENTS,
             ..Default::default()
         },
-        instance_limits: InstanceLimits {
-            count: 1,
-            memory_reservation_size: 1,
-        },
+        instance_limits: InstanceLimits { count: 1 },
     });
+    config.dynamic_memory_guard_size(0);
+    config.static_memory_guard_size(0);
+    config.static_memory_maximum_size(65536);
 
     let engine = Engine::new(&config)?;
 
@@ -357,8 +361,11 @@ fn table_init() -> Result<()> {
 }
 
 #[test]
-#[cfg_attr(target_arch = "aarch64", ignore)] // https://github.com/bytecodealliance/wasmtime/pull/2518#issuecomment-747280133
 fn table_zeroed() -> Result<()> {
+    if skip_pooling_allocator_tests() {
+        return Ok(());
+    }
+
     let mut config = Config::new();
     config.allocation_strategy(InstanceAllocationStrategy::Pooling {
         strategy: PoolingAllocationStrategy::NextAvailable,
@@ -367,11 +374,11 @@ fn table_zeroed() -> Result<()> {
             table_elements: 10,
             ..Default::default()
         },
-        instance_limits: InstanceLimits {
-            count: 1,
-            memory_reservation_size: 1,
-        },
+        instance_limits: InstanceLimits { count: 1 },
     });
+    config.dynamic_memory_guard_size(0);
+    config.static_memory_guard_size(0);
+    config.static_memory_maximum_size(65536);
 
     let engine = Engine::new(&config)?;
 
@@ -413,9 +420,11 @@ fn instantiation_limit() -> Result<()> {
         },
         instance_limits: InstanceLimits {
             count: INSTANCE_LIMIT,
-            memory_reservation_size: 1,
         },
     });
+    config.dynamic_memory_guard_size(0);
+    config.static_memory_guard_size(0);
+    config.static_memory_maximum_size(65536);
 
     let engine = Engine::new(&config)?;
     let module = Module::new(&engine, r#"(module)"#)?;
@@ -450,3 +459,55 @@ fn instantiation_limit() -> Result<()> {
 
     Ok(())
 }
+
+#[test]
+fn preserve_data_segments() -> Result<()> {
+    let mut config = Config::new();
+    config.allocation_strategy(InstanceAllocationStrategy::Pooling {
+        strategy: PoolingAllocationStrategy::NextAvailable,
+        module_limits: ModuleLimits {
+            memory_pages: 1,
+            table_elements: 10,
+            ..Default::default()
+        },
+        instance_limits: InstanceLimits { count: 2 },
+    });
+    let engine = Engine::new(&config)?;
+    let m = Module::new(
+        &engine,
+        r#"
+            (module
+                (memory (export "mem") 1 1)
+                (data (i32.const 0) "foo"))
+        "#,
+    )?;
+    let mut store = Store::new(&engine, ());
+    let i = Instance::new(&mut store, &m, &[])?;
+
+    // Drop the module. This should *not* drop the actual data referenced by the
+    // module, especially when uffd is enabled. If uffd is enabled we'll lazily
+    // fault in the memory of the module, which means it better still be alive
+    // after we drop this.
+    drop(m);
+
+    // Spray some stuff on the heap. If wasm data lived on the heap this should
+    // paper over things and help us catch use-after-free here if it would
+    // otherwise happen.
+    let mut strings = Vec::new();
+    for _ in 0..1000 {
+        let mut string = String::new();
+        for _ in 0..1000 {
+            string.push('g');
+        }
+        strings.push(string);
+    }
+    drop(strings);
+
+    let mem = i.get_memory(&mut store, "mem").unwrap();
+
+    // This will segfault with uffd enabled, and then the uffd will lazily
+    // initialize the memory. Hopefully it's still `foo`!
+    assert!(mem.data(&store).starts_with(b"foo"));
+
+    Ok(())
+}
